import os
import collections
from itertools import count
import datetime
from typing import List, Tuple, Union

lesson_time = ((datetime.time(8, 0), datetime.time(8, 45)),
               (datetime.time(8, 55), datetime.time(9, 40)),
               (datetime.time(10, 0), datetime.time(10, 45)),
               (datetime.time(10, 55), datetime.time(11, 40)),
               (datetime.time(12, 0), datetime.time(12, 45)),
               (datetime.time(12, 55), datetime.time(13, 40)),
               (datetime.time(14, 0), datetime.time(14, 45)),
               (datetime.time(14, 55), datetime.time(15, 40)),
               (datetime.time(16, 0), datetime.time(16, 45)),
               (datetime.time(16, 55), datetime.time(17, 40)),
               (datetime.time(18, 0), datetime.time(18, 45)),
               (datetime.time(18, 55), datetime.time(19, 40)),
               (datetime.time(19, 55), datetime.time(20, 40)),
               (datetime.time(20, 55), datetime.time(21, 40)))


def get_lesson_time(time: Union[int, List[int]]) -> Tuple[datetime.time, datetime.time]:
    if isinstance(time, int):
        return lesson_time[time - 1]
    elif isinstance(time, list):
        return lesson_time[time[0] - 1][0], lesson_time[time[-1] - 1][1]


def get_default_year_term() -> Tuple[int, int]:
    now = datetime.datetime.now()
    _year, month = now.year, now.month
    year = _year - 1 if _year < 7 else _year
    if month < 6:
        term = 1
    elif month < 8:
        term = 2
    else:
        term = 0
    return year, term


def get_available_filename(filename: str) -> str:
    if not os.path.exists(filename):
        return filename
    fn, ext = os.path.splitext(filename)
    for num in count(1):
        fn = f"{fn}_{num}{ext}"
        if not os.path.exists(fn):
            return fn


def flatten(obj):
    for el in obj:
        if isinstance(el, collections.abc.Iterable) and not isinstance(el, (str, bytes)):
            yield from flatten(el)
        else:
            yield el



import datetime
from typing import List

import arrow
from ics import Calendar, Event

from pysjtu.models import ScheduleCourse
from pysjtu.utils import flatten
from .utils import get_lesson_time


def schedule_to_ics(schedule, term_start_date: datetime.date) -> str:
    def lesson_to_events(lesson: ScheduleCourse) -> List[Event]:
        def lesson_week_to_event(week: int):
            lesson_day = term_start_date + datetime.timedelta(days=lesson.day + (week - 1) * 7 - 1)
            begin_time, end_time = get_lesson_time(list(lesson.time))
            e = Event()
            e.name = lesson.name
            e.begin = arrow.get(datetime.datetime.combine(lesson_day, begin_time, local_tz))
            e.end = arrow.get(datetime.datetime.combine(lesson_day, end_time, local_tz))
            e.location = lesson.location
            e.description = lesson.remark
            return e

        return [lesson_week_to_event(week) for week in list(flatten(lesson.week))]

    c = Calendar()
    local_tz = datetime.datetime.now(datetime.timezone.utc).astimezone().tzinfo
    events = set(flatten(lesson_to_events(lesson) for lesson in schedule))
    c.events.update(events)
    return str(c)



import getpass

import blessed
import pysjtu
from pysjtu import Client, JCSSRecognizer, Session
from pysjtu.exceptions import LoginException, OCRException

from . import __version__
from .calendar import schedule_to_ics
from .tui import bool_input, date_picker, selector
from .utils import get_available_filename, get_default_year_term


def banner(term) -> str:
    lines = [f"SJTU Schedule Exporter CLI, Version {__version__}, powered by PySJTU {pysjtu.__version__}",
             f"If you find this tool useful, please star this repo. ",
             f"You are welcomed to submit new issues and PRs if you find any bug in this tool or PySJTU.",
             "",
             f"This project: {term.blue('https://github.com/PhotonQuantum/sjtu-scheudule-exporter-cli')}",
             f"PySJTU: {term.blue('https://github.com/PhotonQuantum/pysjtu')}",
             ""]
    return "\n".join(lines)


def main():
    try:
        sess_file = open("session", mode="r+b")
    except FileNotFoundError:
        sess_file = None

    term = blessed.Terminal()
    print(banner(term))

    print(term.blue("Trying to log into iSJTU..."))
    try:
        with Session(session_file=sess_file, ocr=JCSSRecognizer()) if sess_file else Session(
                ocr=JCSSRecognizer()) as sess:
            if not sess_file:
                username = input("Username: ")
                password = getpass.getpass("Password: ")

                print(term.blue("Logging in..."))
                try:
                    sess.login(username, password)
                except LoginException:
                    print(term.red("Unable to log into iSJTU. Wrong username or password?"))
                except OCRException as e:
                    print(
                        term.red("Failed to solve captcha. Please report the bug and attach the following traceback."))
                    print(str(e))

            client = Client(sess)
            print(term.green(f"Logged in to iSJTU as {client.student_id}."))
            print()

            default_year, default_term = get_default_year_term()
            year_items = list(reversed(range(default_year - 3, default_year + 2)))
            term_items = ["Fall trimester", "Spring trimester", "Summer trimester"]
            year = year_items[selector(term, "Academic year", year_items, 1)]
            print(f"Academic year: {year}")
            trimester = selector(term, "Academic term", term_items, default_term)
            print(f"Academic term: {term_items[trimester]}")
            print()

            print(term.blue("Fetching schedule..."))
            schedule = client.schedule(year, trimester)
            print()

            print(term.green("Just one more step!"))
            validator = lambda date: "A term must start in Monday" if date.weekday() != 0 else date
            default_term_start_date = client.term_start_date
            term_start_date = date_picker(term, "Term start date", default_term_start_date, validator)
            print()

            ics_content = schedule_to_ics(schedule, term_start_date)
            filename = get_available_filename("schedule.ics")
            with open(filename, mode="w") as f:
                f.write(ics_content)

            print(term.green("Done."))
            print()

            if not sess_file:
                save_session = bool_input(term, "Do you want to save your session?")
                if save_session:
                    with open("session", mode="w+b") as f:
                        sess.dump(f)
                    print(term.green(
                        "Saved. You may clear your session at any time by deleting 'session' file in this directory."))
                else:
                    sess.logout()

    except KeyboardInterrupt:
        print(term.red("\nKeyboard interrupted."))


if __name__ == "__main__":
    main()



from typing import Callable, List, Union
import datetime


def print_list(term, title: str, items: List[str], selitem: int, page_count: int):
    if selitem > len(items) - 1:
        selitem = len(items) - 1
    elif selitem < 0:
        selitem = 0
    page = selitem // page_count

    output_msgs = [term.center(term.bright_green(title))]
    with term.location(0, 2):
        for i in range(page * page_count, (page + 1) * page_count):
            if i < len(items):
                if i == selitem:
                    output_msgs.append(term.center(term.underline_bright_cyan(f"{i + 1}. {items[i]}")))
                else:
                    output_msgs.append(term.center(f"{i + 1}. {items[i]}"))
        output_msgs.append(term.clear_eos())
        print("\n".join(output_msgs))
    return selitem


def selector(term, title: str, items: List[str], default_selitem: int = 0) -> int:
    selitem = default_selitem
    with term.hidden_cursor(), term.fullscreen():
        while True:
            selitem = print_list(term, title, items, selitem, 30)
            with term.cbreak():
                val = term.inkey(timeout=1)
                if str(val) != "":
                    if str(val) == "q":
                        raise KeyboardInterrupt
                    elif str(val) == "j" or val.name == "KEY_DOWN":
                        selitem += 1
                    elif str(val) == "k" or val.name == "KEY_UP":
                        selitem -= 1
                    elif str(val) == "d":
                        selitem += 30
                    elif str(val) == "u":
                        selitem -= 30
                    elif val.name == "KEY_ENTER":
                        return selitem


default_validator = lambda x: ""


def date_picker(term, prompt: str, default_date: datetime.date,
                validator: Callable[[datetime.date], str] = default_validator) -> datetime.date:
    def validate_date_str(string: str) -> Union[str, datetime.date]:
        try:
            date = datetime.datetime.strptime(string, "%Y-%m-%d").date() if string else default_date
        except ValueError:
            return "Not a valid date."
        return validator(date) or date

    default_date_display = default_date.strftime("%Y-%m-%d")
    disp = f"{prompt} [{default_date_display}]: "
    raw_input = input(disp)
    date_or_error = validate_date_str(raw_input)

    while not isinstance(date_or_error, datetime.date):
        raw_input = input(term.red(f'{disp}({date_or_error}) '))
        date_or_error = validate_date_str(raw_input)

    return date_or_error


def bool_input(term, prompt: str) -> bool:
    disp = f"{prompt} [y/n]: "
    raw_input = input(disp)
    while raw_input not in ["y", "n"]:
        raw_input = input(term.red(disp))
    return raw_input == "y"
